@@ -23,21 +23,19 @@
  *   http://www.mozilla.org/MPL/                                           *
  ***************************************************************************/
 
-#ifdef HAVE_CONFIG_H
-#include <config.h>
-#endif
+#include <algorithm>
 
 #include <tfile.h>
+#include <tbytevector.h>
+#include <tpropertymap.h>
+#include <tdebug.h>
 
 #include "id3v2tag.h"
 #include "id3v2header.h"
 #include "id3v2extendedheader.h"
 #include "id3v2footer.h"
 #include "id3v2synchdata.h"
-#include "tbytevector.h"
 #include "id3v1genres.h"
-#include "tpropertymap.h"
-#include <tdebug.h>
 
 #include "frames/textidentificationframe.h"
 #include "frames/commentsframe.h"
@@ -49,38 +47,46 @@
 using namespace TagLib;
 using namespace ID3v2;
 
+namespace
+{
+  const ID3v2::Latin1StringHandler defaultStringHandler;
+  const ID3v2::Latin1StringHandler *stringHandler = &defaultStringHandler;
+
+  const long MinPaddingSize = 1024;
+  const long MaxPaddingSize = 1024 * 1024;
+}
+
 class ID3v2::Tag::TagPrivate
 {
 public:
-  TagPrivate() : file(0), tagOffset(-1), extendedHeader(0), footer(0), paddingSize(0)
+  TagPrivate() :
+    file(0),
+    tagOffset(0),
+    extendedHeader(0),
+    footer(0)
   {
     frameList.setAutoDelete(true);
   }
+
   ~TagPrivate()
   {
     delete extendedHeader;
     delete footer;
   }
 
+  const FrameFactory *factory;
+
   File *file;
   long tagOffset;
-  const FrameFactory *factory;
 
   Header header;
   ExtendedHeader *extendedHeader;
   Footer *footer;
 
-  int paddingSize;
-
   FrameListMap frameListMap;
   FrameList frameList;
-
-  static const Latin1StringHandler *stringHandler;
 };
 
-static const Latin1StringHandler defaultStringHandler;
-const ID3v2::Latin1StringHandler *ID3v2::Tag::TagPrivate::stringHandler = &defaultStringHandler;
-
 ////////////////////////////////////////////////////////////////////////////////
 // StringHandler implementation
 ////////////////////////////////////////////////////////////////////////////////
@@ -102,20 +108,20 @@ String Latin1StringHandler::parse(const ByteVector &data) const
 // public members
 ////////////////////////////////////////////////////////////////////////////////
 
-ID3v2::Tag::Tag() : TagLib::Tag()
+ID3v2::Tag::Tag() :
+  TagLib::Tag(),
+  d(new TagPrivate())
 {
-  d = new TagPrivate;
   d->factory = FrameFactory::instance();
 }
 
 ID3v2::Tag::Tag(File *file, long tagOffset, const FrameFactory *factory) :
-  TagLib::Tag()
+  TagLib::Tag(),
+  d(new TagPrivate())
 {
-  d = new TagPrivate;
-
+  d->factory = factory;
   d->file = file;
   d->tagOffset = tagOffset;
-  d->factory = factory;
 
   read();
 }
@@ -125,26 +131,25 @@ ID3v2::Tag::~Tag()
   delete d;
 }
 
-
 String ID3v2::Tag::title() const
 {
   if(!d->frameListMap["TIT2"].isEmpty())
     return d->frameListMap["TIT2"].front()->toString();
-  return String::null;
+  return String();
 }
 
 String ID3v2::Tag::artist() const
 {
   if(!d->frameListMap["TPE1"].isEmpty())
     return d->frameListMap["TPE1"].front()->toString();
-  return String::null;
+  return String();
 }
 
 String ID3v2::Tag::album() const
 {
   if(!d->frameListMap["TALB"].isEmpty())
     return d->frameListMap["TALB"].front()->toString();
-  return String::null;
+  return String();
 }
 
 String ID3v2::Tag::comment() const
@@ -152,7 +157,7 @@ String ID3v2::Tag::comment() const
   const FrameList &comments = d->frameListMap["COMM"];
 
   if(comments.isEmpty())
-    return String::null;
+    return String();
 
   for(FrameList::ConstIterator it = comments.begin(); it != comments.end(); ++it)
   {
@@ -174,7 +179,7 @@ String ID3v2::Tag::genre() const
   if(d->frameListMap["TCON"].isEmpty() ||
      !dynamic_cast<TextIdentificationFrame *>(d->frameListMap["TCON"].front()))
   {
-    return String::null;
+    return String();
   }
 
   // ID3v2.4 lists genres as the fields in its frames field list.  If the field
@@ -208,14 +213,14 @@ String ID3v2::Tag::genre() const
   return genres.toString();
 }
 
-TagLib::uint ID3v2::Tag::year() const
+unsigned int ID3v2::Tag::year() const
 {
   if(!d->frameListMap["TDRC"].isEmpty())
     return d->frameListMap["TDRC"].front()->toString().substr(0, 4).toInt();
   return 0;
 }
 
-TagLib::uint ID3v2::Tag::track() const
+unsigned int ID3v2::Tag::track() const
 {
   if(!d->frameListMap["TRCK"].isEmpty())
     return d->frameListMap["TRCK"].front()->toString().toInt();
@@ -279,7 +284,7 @@ void ID3v2::Tag::setGenre(const String &s)
 #endif
 }
 
-void ID3v2::Tag::setYear(uint i)
+void ID3v2::Tag::setYear(unsigned int i)
 {
   if(i <= 0) {
     removeFrames("TDRC");
@@ -288,7 +293,7 @@ void ID3v2::Tag::setYear(uint i)
   setTextFrame("TDRC", String::number(i));
 }
 
-void ID3v2::Tag::setTrack(uint i)
+void ID3v2::Tag::setTrack(unsigned int i)
 {
   if(i <= 0) {
     removeFrames("TRCK");
@@ -356,7 +361,7 @@ void ID3v2::Tag::removeFrame(Frame *frame, bool del)
 void ID3v2::Tag::removeFrames(const ByteVector &id)
 {
   FrameList l = d->frameListMap[id];
-  for(FrameList::Iterator it = l.begin(); it != l.end(); ++it)
+  for(FrameList::ConstIterator it = l.begin(); it != l.end(); ++it)
     removeFrame(*it, true);
 }
 
@@ -461,15 +466,23 @@ ByteVector ID3v2::Tag::render() const
 
 void ID3v2::Tag::downgradeFrames(FrameList *frames, FrameList *newFrames) const
 {
+#ifdef NO_ITUNES_HACKS
   const char *unsupportedFrames[] = {
     "ASPI", "EQU2", "RVA2", "SEEK", "SIGN", "TDRL", "TDTG",
     "TMOO", "TPRO", "TSOA", "TSOT", "TSST", "TSOP", 0
   };
+#else
+  // iTunes writes and reads TSOA, TSOT, TSOP to ID3v2.3.
+  const char *unsupportedFrames[] = {
+    "ASPI", "EQU2", "RVA2", "SEEK", "SIGN", "TDRL", "TDTG",
+    "TMOO", "TPRO", "TSST", 0
+  };
+#endif
   ID3v2::TextIdentificationFrame *frameTDOR = 0;
   ID3v2::TextIdentificationFrame *frameTDRC = 0;
   ID3v2::TextIdentificationFrame *frameTIPL = 0;
   ID3v2::TextIdentificationFrame *frameTMCL = 0;
-  for(FrameList::Iterator it = d->frameList.begin(); it != d->frameList.end(); it++) {
+  for(FrameList::ConstIterator it = d->frameList.begin(); it != d->frameList.end(); it++) {
     ID3v2::Frame *frame = *it;
     ByteVector frameID = frame->header()->frameID();
     for(int i = 0; unsupportedFrames[i]; i++) {
@@ -535,14 +548,14 @@ void ID3v2::Tag::downgradeFrames(FrameList *frames, FrameList *newFrames) const
     StringList people;
     if(frameTMCL) {
       StringList v24People = frameTMCL->fieldList();
-      for(uint i = 0; i + 1 < v24People.size(); i += 2) {
+      for(unsigned int i = 0; i + 1 < v24People.size(); i += 2) {
         people.append(v24People[i]);
         people.append(v24People[i+1]);
       }
     }
     if(frameTIPL) {
       StringList v24People = frameTIPL->fieldList();
-      for(uint i = 0; i + 1 < v24People.size(); i += 2) {
+      for(unsigned int i = 0; i + 1 < v24People.size(); i += 2) {
         people.append(v24People[i]);
         people.append(v24People[i+1]);
       }
@@ -553,7 +566,6 @@ void ID3v2::Tag::downgradeFrames(FrameList *frames, FrameList *newFrames) const
   }
 }
 
-
 ByteVector ID3v2::Tag::render(int version) const
 {
   // We need to render the "tag data" first so that we have to correct size to
@@ -561,8 +573,6 @@ ByteVector ID3v2::Tag::render(int version) const
   // in ID3v2::Header::tagSize() -- includes the extended header, frames and
   // padding, but does not include the tag's header or footer.
 
-  ByteVector tagData;
-
   if(version != 3 && version != 4) {
     debug("Unknown ID3v2 version, using ID3v2.4");
     version = 4;
@@ -570,7 +580,7 @@ ByteVector ID3v2::Tag::render(int version) const
 
   // TODO: Render the extended header.
 
-  // Loop through the frames rendering them and adding them to the tagData.
+  // Downgrade the frames that ID3v2.3 doesn't support.
 
   FrameList newFrames;
   newFrames.setAutoDelete(true);
@@ -583,48 +593,74 @@ ByteVector ID3v2::Tag::render(int version) const
     downgradeFrames(&frameList, &newFrames);
   }
 
-  for(FrameList::Iterator it = frameList.begin(); it != frameList.end(); it++) {
+  // Reserve a 10-byte blank space for an ID3v2 tag header.
+
+  ByteVector tagData(Header::size(), '\0');
+
+  // Loop through the frames rendering them and adding them to the tagData.
+
+  for(FrameList::ConstIterator it = frameList.begin(); it != frameList.end(); it++) {
     (*it)->header()->setVersion(version);
     if((*it)->header()->frameID().size() != 4) {
-      debug("A frame of unsupported or unknown type \'"
+      debug("An ID3v2 frame of unsupported or unknown type \'"
           + String((*it)->header()->frameID()) + "\' has been discarded");
       continue;
     }
-    if(!(*it)->header()->tagAlterPreservation())
-      tagData.append((*it)->render());
+    if(!(*it)->header()->tagAlterPreservation()) {
+      const ByteVector frameData = (*it)->render();
+      if(frameData.size() == Frame::headerSize((*it)->header()->version())) {
+        debug("An empty ID3v2 frame \'"
+          + String((*it)->header()->frameID()) + "\' has been discarded");
+        continue;
+      }
+      tagData.append(frameData);
+    }
   }
 
   // Compute the amount of padding, and append that to tagData.
+  // TODO: Should be calculated in long long in taglib2.
 
-  uint paddingSize = 0;
-  uint originalSize = d->header.tagSize();
+  long originalSize = d->header.tagSize();
+  long paddingSize = originalSize - (tagData.size() - Header::size());
 
-  if(tagData.size() < originalSize)
-    paddingSize = originalSize - tagData.size();
-  else
-    paddingSize = 1024;
+  if(paddingSize <= 0) {
+    paddingSize = MinPaddingSize;
+  }
+  else {
+    // Padding won't increase beyond 1% of the file size or 1MB.
 
-  tagData.append(ByteVector(paddingSize, char(0)));
+    long threshold = d->file ? d->file->length() / 100 : 0;
+    threshold = std::max(threshold, MinPaddingSize);
+    threshold = std::min(threshold, MaxPaddingSize);
+
+    if(paddingSize > threshold)
+      paddingSize = MinPaddingSize;
+  }
+
+  tagData.resize(static_cast<unsigned int>(tagData.size() + paddingSize), '\0');
 
   // Set the version and data size.
   d->header.setMajorVersion(version);
-  d->header.setTagSize(tagData.size());
+  d->header.setTagSize(tagData.size() - Header::size());
 
   // TODO: This should eventually include d->footer->render().
-  return d->header.render() + tagData;
+  const ByteVector headerData = d->header.render();
+  std::copy(headerData.begin(), headerData.end(), tagData.begin());
+
+  return tagData;
 }
 
 Latin1StringHandler const *ID3v2::Tag::latin1StringHandler()
 {
-  return TagPrivate::stringHandler;
+  return stringHandler;
 }
 
 void ID3v2::Tag::setLatin1StringHandler(const Latin1StringHandler *handler)
 {
   if(handler)
-    TagPrivate::stringHandler = handler;
+    stringHandler = handler;
   else
-    TagPrivate::stringHandler = &defaultStringHandler;
+    stringHandler = &defaultStringHandler;
 }
 
 ////////////////////////////////////////////////////////////////////////////////
@@ -633,18 +669,43 @@ void ID3v2::Tag::setLatin1StringHandler(const Latin1StringHandler *handler)
 
 void ID3v2::Tag::read()
 {
-  if(d->file && d->file->isOpen()) {
+  if(!d->file)
+    return;
 
-    d->file->seek(d->tagOffset);
-    d->header.setData(d->file->readBlock(Header::size()));
+  if(!d->file->isOpen())
+    return;
 
-    // if the tag size is 0, then this is an invalid tag (tags must contain at
-    // least one frame)
+  d->file->seek(d->tagOffset);
+  d->header.setData(d->file->readBlock(Header::size()));
 
-    if(d->header.tagSize() == 0)
-      return;
+  // If the tag size is 0, then this is an invalid tag (tags must contain at
+  // least one frame)
 
+  if(d->header.tagSize() != 0)
     parse(d->file->readBlock(d->header.tagSize()));
+
+  // Look for duplicate ID3v2 tags and treat them as an extra blank of this one.
+  // It leads to overwriting them with zero when saving the tag.
+
+  // This is a workaround for some faulty files that have duplicate ID3v2 tags.
+  // Unfortunately, TagLib itself may write such duplicate tags until v1.10.
+
+  unsigned int extraSize = 0;
+
+  while(true) {
+
+    d->file->seek(d->tagOffset + d->header.completeTagSize() + extraSize);
+
+    const ByteVector data = d->file->readBlock(Header::size());
+    if(data.size() < Header::size() || !data.startsWith(Header::fileIdentifier()))
+      break;
+
+    extraSize += Header(data).completeTagSize();
+  }
+
+  if(extraSize != 0) {
+    debug("ID3v2::Tag::read() - Duplicate ID3v2 tags found.");
+    d->header.setTagSize(d->header.tagSize() + extraSize);
   }
 }
 
@@ -655,8 +716,8 @@ void ID3v2::Tag::parse(const ByteVector &origData)
   if(d->header.unsynchronisation() && d->header.majorVersion() <= 3)
     data = SynchData::decode(data);
 
-  uint frameDataPosition = 0;
-  uint frameDataLength = data.size();
+  unsigned int frameDataPosition = 0;
+  unsigned int frameDataLength = data.size();
 
   // check for extended header
 
@@ -692,8 +753,7 @@ void ID3v2::Tag::parse(const ByteVector &origData)
         debug("Padding *and* a footer found.  This is not allowed by the spec.");
       }
 
-      d->paddingSize = frameDataLength - frameDataPosition;
-      return;
+      break;
     }
 
     Frame *frame = d->factory->createFrame(data.mid(frameDataPosition),
@@ -712,6 +772,8 @@ void ID3v2::Tag::parse(const ByteVector &origData)
     frameDataPosition += frame->size() + Frame::headerSize(d->header.majorVersion());
     addFrame(frame);
   }
+
+  d->factory->rebuildAggregateFrames(this);
 }
 
 void ID3v2::Tag::setTextFrame(const ByteVector &id, const String &value)
